pub type Env = Vec<Defn>;

pub type Label = String;
pub type TpName = String;
pub type VarName = String;
pub type ExpName = String;

#[derive(Debug, PartialEq, Eq, Clone)]
pub enum Type {
    TpName(TpName),
    One,
    Times(Box<Type>, Box<Type>),
    Plus(Vec<(Label, Type)>),
}

#[derive(Debug, PartialEq, Eq)]
pub enum Pat {
    InjPat(Label, Box<Pat>),
    PairPat(Box<Pat>, Box<Pat>),
    UnitPat,
    VarPat(VarName),
}

#[derive(Debug, PartialEq, Eq)]
pub enum Exp {
    Var(VarName),
    Pair(Box<Exp>, Box<Exp>),
    Unit,
    Inj(Label, Box<Exp>),
    MatchWith(Box<Exp>, Vec<(Pat, Exp)>),
    Call(VarName, Vec<Exp>)
    }

#[derive(Debug, PartialEq, Eq)]
pub enum Defn {
    TypeDefn(TpName, Type),
    ExpDefn(ExpName, Vec<(VarName, Type)>, Type, Exp),
}
